from django.contrib import admin
from django.urls import reverse
from django.utils import timezone
from django.utils.html import format_html

from .forms import GenericCurationAdminForm
from .models import AbstractCuration
from .utils import dynamic_short_description
from .utils import get_related_field_name_for_object


class GenericCurationModelAdmin(admin.ModelAdmin):
    form = GenericCurationAdminForm
    list_display = (
        "id",
        "curation_object_link",
        "status",
        "help_required",
        "notice",
        "last_action",
        "curator",
        "date_curated",
    )
    list_filter = ("status",)
    readonly_fields = (
        "status",
        "curation_object_link",
        "initiator",
        "date_created",
        "date_updated",
        "curator",
        "last_action_display",
        "date_curated",
    )

    fieldsets = (
        (
            "System",
            {
                "fields": (
                    "date_created",
                    "date_updated",
                    "initiator",
                    "curation_object_link",
                    "status",
                ),
            },
        ),
        (
            "Curation notes",
            {
                "fields": (
                    "help_required",
                    "notice",
                ),
            },
        ),
        ("Curation controls", {"fields": ("curation_choice", "feedback")}),
        (
            "Last curator",
            {
                "fields": (
                    "curator",
                    "last_action_display",
                    "date_curated",
                ),
            },
        ),
    )

    @dynamic_short_description
    def curation_object_link(self, obj):
        # Generate the URL for the related object's admin change page
        url = reverse(
            f"admin:{obj.curation_object._meta.app_label}_{obj.curation_object._meta.model_name}_change",
            args=[obj.curation_object.pk],
        )
        return format_html('<a href="{}">{}</a>', url, obj.curation_object)

    @admin.display(description="Curator action")
    def last_action_display(self, obj):
        # Create a dictionary from the CURATION_STATUS_CHOICES
        status_dict = dict(AbstractCuration.CURATION_STATUS_CHOICES)

        if obj.curator_action == AbstractCuration.CURATION_PENDING:
            return "To be reviewed"

        # Lookup the display value using the status dictionary
        status = status_dict.get(obj.curator_action, None)
        if not status:
            status = "-"
        return status

    def get_last_action_description(self, obj):
        if obj.curator_action == AbstractCuration.CURATION_PENDING:
            return AbstractCuration.ACTION_TEXT_PENDING

        if obj.curator_action == AbstractCuration.CURATION_ACCEPTED:
            return AbstractCuration.ACTION_TEXT_ACCEPTED

        if obj.curator_action == AbstractCuration.CURATION_REJECTED_TEMPORARILY:
            return AbstractCuration.ACTION_TEXT_REJECTED_TEMPORARILY

        if obj.curator_action == AbstractCuration.CURATION_REJECTED_PERMANENTLY:
            return AbstractCuration.ACTION_TEXT_REJECTED_PERMANENTLY

        if obj.status == AbstractCuration.CURATION_UPDATE_BY_USER:
            return f"Updated by {obj.initiator}"

        return "-"

    def last_action(self, obj):
        return self.get_last_action_description(obj)

    def get_form(self, request, obj=None, **kwargs):
        """Return the form to be used, setting the model dynamically."""
        # Create a dynamic Meta class for our form
        dynamic_meta = type(
            "Meta",
            (GenericCurationAdminForm.Meta,),
            {
                "model": self.model,
            },
        )
        # Create our dynamic form class and set its Meta class
        return type(
            "DynamicForm",
            (GenericCurationAdminForm,),
            {"Meta": dynamic_meta, "request": request},
        )

    def save_model(self, request, obj, form, change):
        """Handle saving the model in the admin panel."""
        curation_choice = form.cleaned_data.get("curation_choice")
        curation_object_attr = get_related_field_name_for_object(
            obj,
            form._curation_object,
        )
        setattr(obj, curation_object_attr, form._curation_object)

        if curation_choice:
            obj.status = curation_choice
            obj.curator = request.user
            obj.curator_action = curation_choice
            obj.date_curated = timezone.now()
            if curation_choice == AbstractCuration.CURATION_ACCEPTED:
                obj.help_required = False

        super().save_model(request, obj, form, change)
