from logging import getLogger

from django import forms

from .models import AbstractCuration

logger = getLogger(__name__)


class GenericCurationAdminForm(forms.ModelForm):
    request = None
    curation_choice = forms.ChoiceField(widget=forms.Select, choices=[], required=False)

    class Meta:
        model = None
        fields = "__all__"  # noqa: DJ007
        widgets = {
            "notice": forms.Textarea(attrs={"rows": 5, "cols": 120}),
            "feedback": forms.Textarea(attrs={"rows": 5, "cols": 120}),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._curation_object = self.instance.curation_object
        self._set_curation_choices()

    def get_staff_permissions(self, request):
        class_name = str(self._curation_object.__class__.__name__).lower()
        can_revert_curation = request.user.has_perm(f"{class_name}.can_revert_curation")
        can_accept_curation = request.user.has_perm(f"{class_name}.can_accept_curation")
        can_reject_curation_permanently = request.user.has_perm(
            f"{class_name}.can_reject_curation_permanently",
        )
        can_reject_curation_temporarily = request.user.has_perm(
            f"{class_name}.can_reject_curation_temporarily",
        )
        allow_actions_on_accepted = request.user.has_perm(
            f"{class_name}.allow_actions_on_accepted",
        )
        allow_actions_on_rejected = request.user.has_perm(
            f"{class_name}.allow_actions_on_rejected",
        )
        return {
            "can_revert_curation": can_revert_curation,
            "can_accept_curation": can_accept_curation,
            "can_reject_curation_permanently": can_reject_curation_permanently,
            "can_reject_curation_temporarily": can_reject_curation_temporarily,
            "allow_actions_on_accepted": allow_actions_on_accepted,
            "allow_actions_on_rejected": allow_actions_on_rejected,
        }

    def _set_curation_choices(self):
        permissions = self.get_staff_permissions(self.request)
        curation_status = self.instance.status
        choices = AbstractCuration.get_next_states(
            curation_status,
            permissions=permissions,
        )

        if not choices:
            self.fields["curation_choice"].widget = forms.HiddenInput()
        else:
            self.fields["curation_choice"].choices = [
                ("", f"currently: {self.instance.get_status_display()}"),
                *choices,
            ]

    def clean(self):
        cleaned_data = super().clean()

        help_required = cleaned_data.get("help_required")
        notice = cleaned_data.get("notice")

        if help_required and not notice:
            self.add_error("notice", "A notice is required when asking for help.")

        for field in self.fields:
            if not cleaned_data.get(field) and self.fields[field].required:
                logger.warning(f"Field '{field}' is required but missing.")

        return cleaned_data
