from django.contrib.auth import get_user_model
from django.db import models

User = get_user_model()

CURATION_RECALLED_BY_USER = "recalled_by_user"
CURATION_PENDING = "pending_review"
CURATION_ACCEPTED = "accepted"
CURATION_REJECTED_TEMPORARILY = "rejected_temporarily"
CURATION_REJECTED_PERMANENTLY = "rejected_permanently"
CURATION_UPDATE_BY_USER = "updated_by_user"  # the state when it was rejected_temporarily but the user has made changes


class AbstractCuration(models.Model):
    CURATION_PENDING = CURATION_PENDING
    CURATION_ACCEPTED = CURATION_ACCEPTED
    CURATION_REJECTED_TEMPORARILY = CURATION_REJECTED_TEMPORARILY
    CURATION_REJECTED_PERMANENTLY = CURATION_REJECTED_PERMANENTLY
    CURATION_UPDATE_BY_USER = CURATION_UPDATE_BY_USER
    CURATION_RECALLED_BY_USER = CURATION_RECALLED_BY_USER

    CURATION_STATUS_CHOICES = [
        (CURATION_PENDING, "Pending Review"),  # initial state by user
        (CURATION_UPDATE_BY_USER, "Pending Review (updated by User)"),
        (CURATION_ACCEPTED, "Accepted"),
        (CURATION_REJECTED_TEMPORARILY, "Rejected Temporarily"),
        (CURATION_REJECTED_PERMANENTLY, "Rejected Permanently"),
        (CURATION_RECALLED_BY_USER, "Recalled by User"),
    ]

    date_updated = models.DateTimeField(auto_now=True)
    date_created = models.DateTimeField(auto_now_add=True)

    # assignee = models.

    status = models.CharField(
        max_length=20,
        choices=CURATION_STATUS_CHOICES,
        default=CURATION_PENDING,
        blank=True,
    )
    initiator = models.ForeignKey(
        User,
        on_delete=models.SET_NULL,
        related_name="%(class)s_initiated_curations",
        null=True,
        blank=True,
    )

    curator = models.ForeignKey(
        User,
        on_delete=models.SET_NULL,
        related_name="%(class)s_edited_curations",
        null=True,
        blank=True,
    )
    curator_action = models.CharField(
        max_length=20,
        choices=CURATION_STATUS_CHOICES,
        blank=True,
        null=True,
    )
    date_curated = models.DateTimeField(blank=True, null=True)

    help_required = models.BooleanField(verbose_name="Require Help", default=False)
    notice = models.CharField(max_length=5000, null=True, blank=True)

    feedback = models.TextField(
        null=True,
        blank=True,
        help_text="This can be displayed to the user for feedback",
    )

    class Meta:
        abstract = True
        permissions = [
            ("can_revert_curation", "Can revert curation"),
            ("can_accept_curation", "Can accept curation"),
            ("can_reject_curation_temporarily", "Can reject curation temporarily"),
            ("can_reject_curation_permanently", "Can reject curation permanently"),
            ("allow_actions_on_accepted", "Allow actions on accepted"),
            ("allow_actions_on_rejected", "Allow actions on rejected"),
        ]

    def __str__(self):
        return f"{self._meta.verbose_name} of {self.curation_object} ({self.pk})"

    @property
    def is_accepted(self):
        return self.status == CURATION_ACCEPTED

    @property
    def show_feedback(self):
        return self.feedback and self.status != CURATION_ACCEPTED

    @property
    def curation_object(self):
        """Return the object associated with this curation."""
        msg = f"{self.__class__.__name__} must implement the 'curation_object' property."
        raise NotImplementedError(msg)

    ACTION_TEXT_ACCEPTED = "Approve"
    ACTION_TEXT_REJECTED_TEMPORARILY = "Reject Temporarily (allow user changes)"
    ACTION_TEXT_REJECTED_PERMANENTLY = "Reject Permanently"
    ACTION_TEXT_PENDING = "Revert to pending"

    BUTTON_CURATION_ACCEPT = (CURATION_ACCEPTED, ACTION_TEXT_ACCEPTED)
    BUTTON_CURATION_REJECT_TEMPORARILY = (
        CURATION_REJECTED_TEMPORARILY,
        ACTION_TEXT_REJECTED_TEMPORARILY,
    )
    BUTTON_CURATION_REJECT_PERMANENTLY = (
        CURATION_REJECTED_PERMANENTLY,
        ACTION_TEXT_REJECTED_PERMANENTLY,
    )
    BUTTON_CURATION_PENDING = (CURATION_PENDING, ACTION_TEXT_PENDING)

    @classmethod
    def get_next_states(cls, current_status, permissions=None):  # noqa: C901, PLR0912
        """Return the possible next states based on current status."""
        next_states = {
            cls.CURATION_PENDING: [],
            cls.CURATION_ACCEPTED: [],
            cls.CURATION_REJECTED_TEMPORARILY: [],
            cls.CURATION_REJECTED_PERMANENTLY: [],
            cls.CURATION_UPDATE_BY_USER: [],
        }

        if current_status == cls.CURATION_PENDING:
            # Possible Buttons for Pending...
            if permissions.get("can_accept_curation"):
                next_states[current_status].append(cls.BUTTON_CURATION_ACCEPT)
            if permissions.get("can_reject_curation_temporarily"):
                next_states[current_status].append(
                    cls.BUTTON_CURATION_REJECT_TEMPORARILY,
                )
            if permissions.get("can_reject_curation_permanently"):
                next_states[current_status].append(
                    cls.BUTTON_CURATION_REJECT_PERMANENTLY,
                )

        if current_status == cls.CURATION_ACCEPTED:
            # Possible Buttons for Accepted...
            if permissions.get("allow_actions_on_accepted"):
                if permissions.get("can_revert_curation"):
                    next_states[current_status].append(cls.BUTTON_CURATION_PENDING)
                if permissions.get("can_reject_curation_temporarily"):
                    next_states[current_status].append(
                        cls.BUTTON_CURATION_REJECT_TEMPORARILY,
                    )
                if permissions.get("can_reject_curation_permanently"):
                    next_states[current_status].append(
                        cls.BUTTON_CURATION_REJECT_PERMANENTLY,
                    )

        if current_status == cls.CURATION_REJECTED_TEMPORARILY:
            if permissions.get("allow_actions_on_rejected"):
                # Possible Buttons for Rejected Temporarily...
                if permissions.get("can_revert_curation"):
                    next_states[current_status].append(cls.BUTTON_CURATION_PENDING)
                if permissions.get("can_reject_curation_permanently"):
                    next_states[current_status].append(
                        cls.BUTTON_CURATION_REJECT_PERMANENTLY,
                    )

        if current_status == cls.CURATION_REJECTED_PERMANENTLY:
            if permissions.get("allow_actions_on_rejected"):
                # Possible Buttons for Rejected Permanently...
                if permissions.get("can_revert_curation"):
                    next_states[current_status].append(cls.BUTTON_CURATION_PENDING)
                if permissions.get("can_reject_curation_temporarily"):
                    next_states[current_status].append(
                        cls.BUTTON_CURATION_REJECT_TEMPORARILY,
                    )

        if current_status == cls.CURATION_UPDATE_BY_USER:
            # Possible Buttons for users that have re-submitted the curation...
            if permissions.get("can_accept_curation"):
                next_states[current_status].append(cls.BUTTON_CURATION_ACCEPT)
            if permissions.get("can_reject_curation_temporarily"):
                next_states[current_status].append(
                    cls.BUTTON_CURATION_REJECT_TEMPORARILY,
                )
            if permissions.get("can_reject_curation_permanently"):
                next_states[current_status].append(
                    cls.BUTTON_CURATION_REJECT_PERMANENTLY,
                )

        return next_states.get(current_status, [])
